/*

 Author: 
  Sebastien Verel, 
  Univ. du Littoral Côte d'Opale, France.
  
*/

#ifndef _batchGradientDescent_h
#define _batchGradientDescent_h

#include <vector>

#include <gradientDescent.h>
#include <lossFunction.h>

class BatchGradientDescent : public GradientDescent {
public:
    BatchGradientDescent(LossFunction & _loss, double _sigma, unsigned _nIter) : loss(_loss), sigma(_sigma), nIter(_nIter) {

    }

    virtual void optim(std::vector<double> & x) {
        std::vector<double> grad(x.size());

        for(unsigned iter = 0; iter < nIter; iter++) {
            loss.gradient(x, grad);

            // x = x - sigma * grad f
            for(unsigned k = 0; k < x.size(); k++) {
                x[k] -= sigma * grad[k];
            }

        }
    }

protected:
    LossFunction & loss;

    double sigma;

    unsigned nIter;

};

#endif
