#ifndef _LossFunction_h
#define _LossFunction_h

#include <data.h>

/*
  Loss function (Mean Square Error) for multi-linear models
 */
class LossFunction {
public:
	LossFunction(Data & _data) : data(_data) {

	}

	/* 
		Mean Square Error (MSE) on all data
	*/
	double value(std::vector<double> & param) {
		double v = 0;

		for(unsigned i = 0; i < data.x.size(); i++) {
			v += value(param, i);
		}

		return v / data.x.size();
	}

	/* 
		Mean Square Error (MSE) on the indexed data
	*/
	double value(std::vector<double> & param, std::vector<unsigned> & index) {
		double v = 0;

		for(unsigned i = 0; i < index.size(); i++) {
			v += value(param, index[i]);
		}

		return v / index.size();
	}

	/* 
		Square Error on the data i
	*/
	virtual double value(std::vector<double> & param, unsigned i) {
		double s = param[0];

		for(unsigned j = 1; j < param.size(); j++) {
			s += param[j] * data.x[i][j-1];
		}

		s = s - data.y[i];

		return 0.5 * s * s;
	}

	void gradient(std::vector<double> & param, std::vector<double> & grad) {
		for(unsigned k = 0; k < grad.size(); k++) {
			grad[k] = 0;
		}

		std::vector<double> gi(param.size(), 0);

		for(unsigned i = 0; i < data.x.size(); i++) {
			gradient(param, i, gi);

			for(unsigned k = 0; k < gi.size(); k++) {
				grad[k] += gi[k];
			}
		}

		for(unsigned k = 0; k < grad.size(); k++) {
			grad[k] /= data.x.size();
		}
	}

	void gradient(std::vector<double> & param, std::vector<unsigned> & index, std::vector<double> & grad) {
		for(unsigned k = 0; k < grad.size(); k++) {
			grad[k] = 0;
		}

		std::vector<double> gi(param.size(), 0);

		for(unsigned i = 0; i < index.size(); i++) {
			gradient(param, index[i], gi);

			for(unsigned k = 0; k < gi.size(); k++) {
				grad[k] += gi[k];
			}
		}

		for(unsigned k = 0; k < grad.size(); k++) {
			grad[k] /= index.size();
		}
	}

	virtual void gradient(std::vector<double> & param, unsigned i, std::vector<double> & grad) {
		// *****
		// To be complete 
		// *****
	}

	Data & data;

};

#endif