/*

 Author: 
  Sebastien Verel, 
  Univ. du Littoral Côte d'Opale, France.

 Date:
   2018/11/21

 Licence:

 See for reference: 
   Sebastian Ruder, An overview of gradient descent optimization algorithms, arXiv:1609.04747

 To compile:
   mkdir build
   cmake ../src
   make

 To execute:
   ./main data1.csv 1000 "0 0.01"
   ./main data1.csv 1000 "1 0.01 4"

*/

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <random>

#include <data.h>
#include <lossFunction.h>
#include <gradientDescent.h>
#include <batchGradientDescent.h>
//#include <stochasticGradientDescent.h>
//#include <momentumGradientDescent.h>

using namespace std;

int main(int argc, char ** argv) {
    //---------------------------------
    // *** Arguments

    // file name of data 
    string filename(argv[1]);

    // number of iteration
    unsigned nIter = atoi(argv[2]);

    // parameters of the gradient descent
    string param_gd(argv[3]);

    //---------------------------------
    // *** Parameters of the gradient descent methods

    // data
    Data data(filename);

    // loss function (value and gradient) for linear model
    LossFunction loss(data);

    // gradient descent method
    GradientDescent * gd;

    istringstream ss_gd(param_gd);

    // id of the method
    unsigned gd_id;
    ss_gd >> gd_id;

    // parameters
    double sigma, gamma;
    long unsigned int seed;
    std::mt19937 gen;

    switch (gd_id) { 
        case 0: // Batch gradient descent
            ss_gd >> sigma;
            gd = new BatchGradientDescent(loss, sigma, nIter);
        break;
        /*
        case 1: // Batch gradient descent
            ss_gd >> sigma >> seed;
            gen.seed(seed);
            gd = new StochasticGradientDescent(gen, loss, sigma, nIter);
        break;
        case 2: // Momentum gradient descent
            ss_gd >> sigma >> gamma;
            gd = new MomentumGradientDescent(loss, sigma, gamma, nIter);
        break;
        */
    }

    //---------------------------------
    // *** Run the Gradient Descent method

    // initial solution
    vector<double> x(2, 0);
    x[0] = data.meanY();

    // run optimization algorithm
    (*gd).optim(x);

    // Print the value of the loss function
    cout << loss.value(x);

    // Print parameters of the regression
    for(unsigned i = 0; i < x.size(); i++)
        cout << " " << x[i] ;
    cout << endl;

    delete gd;

    return 1;
}